/* +---------------------------------------------------------------------------+
   |          The Mobile Robot Programming Toolkit (MRPT) C++ library          |
   |                                                                           |
   |                   http://mrpt.sourceforge.net/                            |
   |                                                                           |
   |   Copyright (C) 2005-2009  University of Malaga                           |
   |                                                                           |
   |    This software was written by the Machine Perception and Intelligent    |
   |      Robotics Lab, University of Malaga (Spain).                          |
   |    Contact: Jose-Luis Blanco  <jlblanco@ctima.uma.es>                     |
   |                                                                           |
   |  This file is part of the MRPT project.                                   |
   |                                                                           |
   |     MRPT is free software: you can redistribute it and/or modify          |
   |     it under the terms of the GNU General Public License as published by  |
   |     the Free Software Foundation, either version 3 of the License, or     |
   |     (at your option) any later version.                                   |
   |                                                                           |
   |   MRPT is distributed in the hope that it will be useful,                 |
   |     but WITHOUT ANY WARRANTY; without even the implied warranty of        |
   |     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         |
   |     GNU General Public License for more details.                          |
   |                                                                           |
   |     You should have received a copy of the GNU General Public License     |
   |     along with MRPT.  If not, see <http://www.gnu.org/licenses/>.         |
   |                                                                           |
   +---------------------------------------------------------------------------+ */

#ifndef SLAMDEMOMAIN_H
#define SLAMDEMOMAIN_H

#define SLAMDEMO_VERSION  "1.0"

//(*Headers(slamdemoFrame)
#include <wx/toolbar.h>
#include <wx/sizer.h>
#include <wx/notebook.h>
#include <wx/menu.h>
#include <mrpt/otherlibs/mathplot/mathplot.h>
#include <wx/panel.h>
#include <wx/statusbr.h>
#include <wx/frame.h>
#include <wx/stattext.h>
#include <wx/timer.h>
//*)

#include <mrpt/core.h>
#include <mrpt/slam/CRangeBearingKFSLAM2D.h>

class slamdemoFrame: public wxFrame
{
    public:

        slamdemoFrame(wxWindow* parent,wxWindowID id = -1);
        virtual ~slamdemoFrame();

    private:

        //(*Handlers(slamdemoFrame)
        void OnQuit(wxCommandEvent& event);
        void OnAbout(wxCommandEvent& event);
        void OnbtnResetClicked(wxCommandEvent& event);
        void OnbtnOneStepClicked(wxCommandEvent& event);
        void OnbtnRunClicked(wxCommandEvent& event);
        void OnbtnStopClicked(wxCommandEvent& event);
        void OnbtnRunBatchClicked(wxCommandEvent& event);
        void OntimSimulTrigger(wxTimerEvent& event);
        void OnConfigClicked(wxCommandEvent& event);
        //*)

        //(*Identifiers(slamdemoFrame)
        static const long ID_STATICTEXT1;
        static const long ID_PANEL3;
        static const long ID_CUSTOM1;
        static const long ID_STATICTEXT2;
        static const long ID_PANEL4;
        static const long ID_CUSTOM2;
        static const long ID_STATICTEXT10;
        static const long ID_PANEL5;
        static const long ID_CUSTOM3;
        static const long ID_STATICTEXT3;
        static const long ID_PANEL6;
        static const long ID_CUSTOM4;
        static const long ID_STATICTEXT4;
        static const long ID_PANEL7;
        static const long ID_CUSTOM5;
        static const long ID_STATICTEXT11;
        static const long ID_PANEL8;
        static const long ID_CUSTOM6;
        static const long ID_STATICTEXT5;
        static const long ID_PANEL9;
        static const long ID_CUSTOM7;
        static const long ID_STATICTEXT6;
        static const long ID_PANEL10;
        static const long ID_CUSTOM8;
        static const long ID_STATICTEXT7;
        static const long ID_PANEL11;
        static const long ID_CUSTOM9;
        static const long ID_PANEL1;
        static const long ID_PANEL2;
        static const long ID_STATICTEXT8;
        static const long ID_PANEL13;
        static const long ID_CUSTOM10;
        static const long ID_PANEL12;
        static const long ID_NOTEBOOK1;
        static const long ID_MENUITEM1;
        static const long ID_MENUITEM2;
        static const long ID_MENUITEM3;
        static const long ID_MENUITEM4;
        static const long ID_MENUITEM5;
        static const long idMenuQuit;
        static const long idMenuAbout;
        static const long ID_STATUSBAR1;
        static const long ID_TOOLBARITEM1;
        static const long ID_TOOLBARITEM2;
        static const long ID_BTNRUN;
        static const long ID_BTNSTOP;
        static const long ID_TOOLBARITEM4;
        static const long ID_TOOLBARITEM3;
        static const long ID_TOOLBARITEM6;
        static const long ID_TOOLBARITEM7;
        static const long ID_TOOLBAR1;
        static const long ID_TIMER1;
        //*)

        //(*Declarations(slamdemoFrame)
        mpWindow* plotErrorY;
        wxPanel* Panel11221;
        wxToolBarToolBase* ToolBarItem5;
        wxPanel* Panel11;
        wxPanel* Panel1;
        wxPanel* Panel6;
        wxStaticText* lmIndCompat;
        mpWindow* plotErrorPhi;
        wxPanel* Panel7;
        wxStatusBar* StatusBar1;
        mpWindow* plotGT;
        wxToolBarToolBase* ToolBarItem6;
        mpWindow* plotIndivCompat;
        wxPanel* Panel12;
        mpWindow* plotMap;
        wxPanel* Panel9;
        wxPanel* Panel8;
        wxStaticText* StaticText1;
        wxPanel* Panel10;
        wxToolBarToolBase* ToolBarItem7;
        wxPanel* Panel2;
        mpWindow* plotErrorX;
        wxToolBarToolBase* ToolBarItem2;
        wxStaticText* StaticText3;
        wxPanel* Panel4;
        wxStaticText* lbDatAssoc;
        wxMenuItem* MenuItem3;
        mpWindow* plotStatTime;
        wxPanel* Panel5;
        wxStaticText* lbMap;
        mpWindow* plotObs;
        wxToolBar* ToolBar1;
        wxPanel* Panel3;
        wxStaticText* lbGT;
        wxToolBarToolBase* ToolBarItem4;
        wxStaticText* lbObs;
        wxStaticText* StaticText4;
        mpWindow* plotDataAssoc;
        wxToolBarToolBase* ToolBarItem1;
        wxTimer timSimul;
        wxStaticText* StaticText2;
        wxNotebook* Notebook1;
        wxToolBarToolBase* ToolBarItem3;
        mpWindow* plotFinalDataAssoc;
        wxStaticText* lbDatAssocFinal;
        wxToolBarToolBase* ToolBarItem8;
        //*)

        DECLARE_EVENT_TABLE()


		// Layers in GT plot -------------
		mpFXYVector	 *m_lyGTMap;
		mpPolygon	 *m_lyGTRobot;
		mpPolygon	 *m_lyGTvisibleRange;

		// Layers in Map plot -------------
		mpPolygon	 *m_lyMapRobot;
		std::vector<mpCovarianceEllipse*> m_lyMapEllipses;

		// Layers in observation plot -------------
		mpPolygon	 *m_lyObsRobot;
		mpPolygon	 *m_lyObsvisibleRange;
		std::vector<mpCovarianceEllipse*> m_lyObsLMs;

		// Layers in errorX plot ----------
		mpFXYVector	 *m_lyERRX_err;
		mpFXYVector	 *m_lyERRX_boundUp;
		mpFXYVector	 *m_lyERRX_boundDown;

		// Layers in errorY plot ----------
		mpFXYVector	 *m_lyERRY_err;
		mpFXYVector	 *m_lyERRY_boundUp;
		mpFXYVector	 *m_lyERRY_boundDown;

		// Layers in errorPhi plot ----------
		mpFXYVector	 *m_lyERRPHI_err;
		mpFXYVector	 *m_lyERRPHI_boundUp;
		mpFXYVector	 *m_lyERRPHI_boundDown;

		// Layers in stats Time plot ----------
		mpFXYVector	 *m_lyStatTimes;


		/** The main object which handles all 2D-EKF SLAM */
		mrpt::slam::CRangeBearingKFSLAM2D		m_SLAM;

		/** The ground truth map, used to simulate observations */
		mrpt::slam::CLandmarksMap				m_GT_map;

		/** The ground truth robot pose, used to simulate observations */
		mrpt::poses::CPose2D					m_GT_pose;

		mrpt::slam::CObservationBearingRange	m_lastObservation;

		/** Historic data */
		struct THistoric
		{
			mrpt::poses::CPose2D            GT_robot_pose;
			mrpt::poses::CPosePDFGaussian   estimate_robot_pose;
			double	run_time;
		};

		std::deque<THistoric>  m_historicData;	//!< A registry of all data for the simulation, used to compute errors, etc..

		/** Reset the simulator and re-generate the ground truth map
		  *  map_type can be:
		  *		- "1": The default map
		  *		- a plain text file name: A Nx2 matrix with the coordinates of the landmarks
		  */
		void resetSimulator( const std::string &map_type );

		void executeOneStep(); //!< Executes 1 step of the simulator (does NOT update the graphs)


		/** Update all the plots with the latest data
		  */
		void updateAllGraphs(bool alsoGTMap = false);


		struct TSimulationOptions
		{
			TSimulationOptions();

			int random_seed;	//!< -1: random, other, use as seed

			double sensor_maxR; //!< Maximum range of the sensor (meters)
			double sensor_FOV;  //!< Field of view of the sensor (rads)


			std::string  map_generator; //!< the parameter to resetSimulator

			unsigned int randomMap_nLMs; //!< # of landmarks in the random map

			mrpt::poses::CPose2D  sensorOnTheRobot;

			bool		sensorDistingishesLandmarks; //!< If false (default), data associatin must be done

			double		path_square_len; //!< Used to simulate the robot path
			double		robot_step_length; //!< The length (in meters) of each robot forward step.

			double		odometry_noise_std_xy; //!< sigma of the odometry errors in X/Y
			double		odometry_noise_std_phi; //!< sigma of the odometry errors in PHI

		};


		TSimulationOptions options; //!< Options used in the simulator




};

#ifdef wxUSE_UNICODE
#define _U(x) wxString((x),wxConvUTF8)
#define _UU(x,y) wxString((x),y)
#else
#define _U(x) (x)
#define _UU(x,y) (x)
#endif


#endif // SLAMDEMOMAIN_H
