/* +---------------------------------------------------------------------------+
   |          The Mobile Robot Programming Toolkit (MRPT) C++ library          |
   |                                                                           |
   |                   http://mrpt.sourceforge.net/                            |
   |                                                                           |
   |   Copyright (C) 2005-2009  University of Malaga                           |
   |                                                                           |
   |    This software was written by the Perception and Robotics               |
   |      research group, University of Malaga (Spain).                        |
   |    Contact: Jose-Luis Blanco  <jlblanco@ctima.uma.es>                     |
   |                                                                           |
   |  This file is part of the MRPT project.                                   |
   |                                                                           |
   |     MRPT is free software: you can redistribute it and/or modify          |
   |     it under the terms of the GNU General Public License as published by  |
   |     the Free Software Foundation, either version 3 of the License, or     |
   |     (at your option) any later version.                                   |
   |                                                                           |
   |   MRPT is distributed in the hope that it will be useful,                 |
   |     but WITHOUT ANY WARRANTY; without even the implied warranty of        |
   |     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         |
   |     GNU General Public License for more details.                          |
   |                                                                           |
   |     You should have received a copy of the GNU General Public License     |
   |     along with MRPT.  If not, see <http://www.gnu.org/licenses/>.         |
   |                                                                           |
   +---------------------------------------------------------------------------+ */

#include <mrpt/core.h>
#include <mrpt/monoslam/CMonoSlam.h>
#include <mrpt/monoslam/CMonoSlamMotionModel.h>

using namespace std;
using namespace mrpt;
using namespace mrpt::utils;
using namespace mrpt::poses;
using namespace mrpt::slam;
using namespace mrpt::monoslam;
using namespace mrpt::math;
using namespace mrpt::gui;
using namespace mrpt::vision;
using namespace mrpt::hwdrivers;


/*****************************************************AJOGD********************************************************************/

void  CMonoSlam::InitializeNewFeature(CFeaturePatch &features, mrpt::utils::CImage &im)
{
	//std::vector<float> r,c;
	float col,row;
	size_t i;
	unsigned int	   max_num_feat_init = options.max_num_feat_to_detect;
	//mrpt::vision::CFeatureList       featList;
	mrpt::vision::CFeatureExtraction feat_extract;

#if 1
	// KLT method:
	feat_extract.options.featsType = featHarris; // featKLT;
	feat_extract.options.KLTOptions.min_distance = options.radius;
#else
	// Harris method:
	feat_extract.options.method=mrpt::vision::CFeatureExtraction::extMethodHarris;
	feat_extract.options.harrisOptions.min_distance = radius;

	//feat_extract.options.harrisOptions.threshold=threshold_ini;
	//feat_extract.options.harrisOptions.radius=radius;//40
	//feat_extract.options.harrisOptions.sigma=sigma;//7
#endif

	feat_extract.options.patchSize = 0; // Do not extract patch, its extract later on below

	// We look for a number of features in the whole image but at the borders, since they might be lost very soon
	// JL: A border is already assumed within the detector, no need to enter a ROI here

	//unsigned int lim_dch = min(im.getWidth()-10,  lim_dch);
	//unsigned int lim_inf = min(im.getHeight()-10,  lim_inf);
	//unsigned int x_inicio = 10;
	//unsigned int y_inicio = 10;
	//unsigned int x_fin = lim_dch;
	//unsigned int y_fin = lim_inf;
 //   TImageROI ROI( x_inicio, x_fin, y_inicio, y_fin);
   //feat_extract.detectFeatures( im, m_featList/*featList*/, 0, max_num_feat_init , ROI );

   feat_extract.detectFeatures( im, m_featList/*featList*/, 0, max_num_feat_init );

#if 0    // DEBUG: Show detected features:
	{
		CDisplayWindow	win_feats;
		win_feats.showImageAndPoints(im,featList);
		cout << "[debug] Showing detected features. Press a key to continue..." << endl;
		win_feats.waitForKey();
	}
#endif

 	for (i=0;i<m_featList.size();i++)
	{
		col=m_featList[i]->x;//+0.5;
		row=m_featList[i]->y;//+0.5;

		//std::cout<<"col detectada: "<<col<<"	row detectada: "<<row<<"\n";
		addNewFeatures_newPar(col,row);
		features.insert_feature();

		features.set_patch((unsigned int)features.size()-1,round(col),round(row),im);
		//features.set_patch((unsigned int)features.size()-1,round(col-0.5),round(row-0.5),im);

		std::cout<<"[Mono-Slam] Initial Feature     #" <<i<< "  x:  "<<col <<"		y: "<<row << "\n";
 
 		//MAAA for test
		//TPoint3D hrl;
	    //TPixelCoordf         vProjUV,vPixel(col,row);
 		//cam.unproject_3D_point(vPixel, hrl);
		//cam.project_3D_point( hrl, vProjUV);
    	//std::cout<<"[Mono-Slam] Projected  x: "<<vProjUV.x <<"	y: "<<vProjUV.y << "\n";

		//CDisplayWindow	win_feats;
		//CImage imgPatch;
		//features.get_patch(i,imgPatch);
		//win_feats.showImage(imgPatch);
		//cout << "[debug] Showing detected features. Press a key to continue..." << endl;
		//win_feats.waitForKey();

	}

		//mrpt::math::CVectorDouble cc,rr;
		//cc.resize(c.size());
		//rr.resize(r.size());
		//for(i =0;i<c.size();i++)
		//{
		//	cc[i] = c[i];
		//	rr[i] = r[i];
		//}
		//cc.saveToTextFile("c:\\csubp.txt");
		//rr.saveToTextFile("c:\\rsubp.txt");

}


/******************************************************************************************************************************/
/*****************************************************AJOGD********************************************************************/

void  CMonoSlam::SearchMoreFeature(CFeaturePatch &features, mrpt::utils::CImage &im, CMatrixDouble &h)
{
	//std::vector<float> r,c;
	mrpt::math::CVectorDouble rz,cz;
	float col,row;
	mrpt::vision::CFeatureList  outfeats;
	mrpt::vision::CFeatureExtraction feat_extract;
	unsigned int num_feat = (unsigned int)(h.getRowCount());

	unsigned int t=0;
	unsigned int border_vert=im.getHeight()/5;
	unsigned int border_horz=im.getHeight()/5;

	cz.resize(num_feat);
	rz.resize(num_feat);
	for (size_t j=0;j<num_feat;j++)
		if ((h(j,0)>0)&&(h(j,1)>0))
		{
			cz[t] = h(j,0);								//(int)
			rz[t] = h(j,1);								//(int)
			t++;
		}

	// JLBC:
	if (!t)
		return; // No need for more feats...

	cz.resize(t);
	rz.resize(t);

	num_feat = (unsigned int)cz.size();

	double min_c,max_c,min_r,max_r;
	size_t indx;
	cz.find_min_max(indx,indx,min_c,max_c);
	rz.find_min_max(indx,indx,min_r,max_r);

	int dif_sup=round(min_r-30);
	int dif_inf=round(im.getHeight()-max_r-30);
	int dif_izq=round(min_c-30);
	int dif_dch=round(im.getWidth()-max_c-30);

	size_t lim_sup = 0;
	size_t lim_inf = 0;
	size_t lim_izq = 0;
	size_t lim_dch = 0;


	if ((dif_sup>=dif_inf)&&(dif_sup>=dif_dch)&&(dif_sup>=dif_izq)&&(dif_sup>0))		//superior
	{
		//std::cout<<"superior\n";
		lim_sup = 10;
		lim_inf = round(min_r - 20);
		lim_izq = 10;
		lim_dch = im.getWidth() - 11;
	}
	else if ((dif_inf>=dif_sup)&&(dif_inf>=dif_dch)&&(dif_inf>=dif_izq)&&(dif_inf>0))	//inferior
	{
		//std::cout<<"inferior\n";
		lim_sup = round(max_r + 20);
		lim_inf = im.getHeight() - 11;
		lim_izq = 10;
		lim_dch = im.getWidth() - 11;
	}
	else if ((dif_izq>=dif_sup)&&(dif_izq>=dif_inf)&&(dif_izq>=dif_dch)&&(dif_izq>0))	//izquierda
	{
		//std::cout<<"izq\n";
		lim_sup = 10;
		lim_inf = im.getHeight() - 11;
		lim_izq = 10;
		lim_dch = round(min_c - 20);
	}
	else if  ((dif_dch>=dif_sup)&&(dif_dch>=dif_inf)&&(dif_dch>=dif_izq)&&(dif_dch>0))	//derecha
	{
		//std::cout<<"dch\n";
		lim_sup = 10;
		lim_inf = im.getHeight() - 11;
		lim_izq = round(max_c+20);
		lim_dch = im.getWidth() - 11;
	}
	//std::cout<<"izq: "<<lim_izq<<"long x: "<<lim_dch-lim_izq<<"\n";
	//std::cout<<"sup: "<<lim_sup<<"long y: "<<lim_inf-lim_sup<<"\n";
	if ((lim_dch-lim_izq > border_horz)&&(lim_inf-lim_sup > border_vert))	//si la zona de busqueda es mas ancha que el pad
	{

	#if 1
		// KLT method:
		feat_extract.options.featsType = featHarris; // featKLT; // ;
		feat_extract.options.KLTOptions.min_distance = options.radius;
	#else
		// Harris method:
		feat_extract.options.method=mrpt::vision::CFeatureExtraction::extMethodHarris;
		feat_extract.options.harrisOptions.min_distance = radius;

		//feat_extract.options.harrisOptions.threshold=threshold_ini;
		//feat_extract.options.harrisOptions.radius=radius;//40
		//feat_extract.options.harrisOptions.sigma=sigma;//7
	#endif
		feat_extract.options.patchSize = 0; 

		//Buscamos caracteristicas hasta llegar a 14 en la zona "mas" vacia de la imagen

		lim_dch = min(im.getWidth()-10,  lim_dch);
		lim_inf = min(im.getHeight()-10,  lim_inf);

		//Buscamos 14 caracteristicas, salvo en el borde xq se perderian enseguida
		unsigned int x_inicio = lim_sup;
		unsigned int y_inicio = lim_izq;
		unsigned int x_fin = lim_dch;
		unsigned int y_fin = lim_inf;

		TImageROI ROI( x_inicio, x_fin, y_inicio, y_fin);

		//std::cout<<"findMoreFeatures\n";

		if(m_ObservedfeatList.size()>0)
		   feat_extract.findMoreFeatures( im, /*m_featList*/m_ObservedfeatList,outfeats,1 );
		else
		   feat_extract.detectFeatures( im, outfeats, 0, 1, ROI );

		std::cout<<" ObservedfeatList.size  :" << m_ObservedfeatList.size()<< " \n";

		size_t ObsSize = get_observation_size();

 		for (size_t i=0;i<outfeats.size();i++)
		{
			col=outfeats[i]->x; //c //+0.5;
			row=outfeats[i]->y; //r //+0.5;

 			if ((col<10) || (col>im.getWidth()-11) || (row<10) || (row>im.getHeight()-11) )
				continue;

			features.insert_feature();
			features.set_patch((unsigned int)features.size()-1,round(col),round(row),im);
			h.setSize(h.getRowCount()+1,ObsSize);

			m_ObservedfeatList.push_back(outfeats[i]);
			m_featList.push_back(outfeats[i]);
			//m_featList.resize(m_featList.size()+1);
			//m_featList[m_featList.size()-1] = outfeats[i];

			addNewFeatures_newPar(col,row);

			//std::cout<<"addNewFeatures_newPar OK\n";

		}
        
	}
}

/******************************************************************************************************************************/
void  CMonoSlam::addNewFeatures_newPar(float &col,float &row)
{
	CMatrixDouble h_LR_rot;
	h_LR_rot.setSize(3,1);
					
    //h_LR_rot = calculate_ray (col,row);

	TPoint3D vRay;
	TPixelCoordf         vPixel(col,row);


	cam.unproject_3D_point(vPixel, vRay);
		
	//std::cout<<"unproject_3D_point OK\n";

	mrpt::math::CVectorDouble newFeature(feature_size);
    h_LR_rot(0,0) = vRay.x;
	h_LR_rot(1,0) = vRay.y;
	h_LR_rot(2,0) = vRay.z; //Should be == 1

	newFeature = hinv(h_LR_rot);

	//std::cout<<"hinv OK\n";

	//std::cout<<"[Mono-Slam] Feature 3DPose:  x: "<<newFeature[0]  <<" 	y: " << newFeature[1] <<"	   z: "<<newFeature[2] << "\n";
 //   std::cout<<"[Mono-Slam] FeatureORNT    Phi: "<<newFeature[3]  <<" 	Thet: " << newFeature[4] <<"   lam: "<<newFeature[5] << "\n";

	unsigned int index;
	index = (unsigned int)m_xkk.size();
	m_xkk.resize(index+newFeature.size());
	m_xkk.insert_vector(index,newFeature);

	std::cout<<"addAFeatureCov_newPar started\n";

	addAFeatureCov_newPar(h_LR_rot,col,row);

	//std::cout<<"addAFeatureCov_newPar OK\n";

}


/******************************************************************************************************************************/
/**	Esta funcion elimina de xkk y pkk la caracteristica con indice "feat"
*/
void CMonoSlam::delete_feature(const unsigned int &index)
{
	//elimination of the feature in the xkk 
 	if (state_size+feature_size*index > m_xkk.size())
		THROW_EXCEPTION("ekf_delete_feature: wrong feature index");

	CVectorDouble first_part;
	CVectorDouble second_part;

	first_part.resize(state_size+feature_size*index);
	second_part.assign(m_xkk.size()-state_size-feature_size*(1+index),0);
	m_xkk.extract_vector(0,first_part);
	m_xkk.extract_vector(state_size+feature_size*(1+index),second_part);
	m_xkk.concatenate(first_part,second_part);

	//elimination of the feature in the pkk
	CMatrixDouble left_up(state_size+feature_size*index, state_size+feature_size*index);
	CMatrixDouble right_up(state_size+feature_size*index, m_pkk.getColCount()-state_size-feature_size*(1+index));
	CMatrixDouble left_down(m_pkk.getRowCount()-state_size-feature_size*(1+index), state_size+feature_size*index);
	CMatrixDouble right_down(m_pkk.getRowCount()-state_size-feature_size*(1+index), m_pkk.getColCount()-state_size-feature_size*(1+index));

	m_pkk.extractMatrix(0,0,left_up);
	m_pkk.extractMatrix(0,state_size+feature_size*(1+index),right_up);
	m_pkk.extractMatrix(state_size+feature_size*(1+index),0,left_down);
	m_pkk.extractMatrix(state_size+feature_size*(1+index),state_size+feature_size*(1+index),right_down);
	m_pkk.joinMatrix(left_up,right_up,left_down,right_down);
 
	//added by MAAA (2009)
	//elimination of the feature in the m_S
	size_t ObsSize = get_observation_size();
	CMatrixDouble left_up_S(ObsSize*index, ObsSize*index);
	CMatrixDouble right_up_S(ObsSize*index, m_S.getColCount()-ObsSize*(1+index));
	CMatrixDouble left_down_S( m_S.getRowCount()- ObsSize*(1+index), ObsSize*index);
	CMatrixDouble right_down_S( m_S.getRowCount()- ObsSize*(1+index), m_S.getColCount()-ObsSize*(1+index));

	m_S.extractMatrix(0,0,left_up_S);
	m_S.extractMatrix(0,ObsSize*(1+index),right_up_S);
	m_S.extractMatrix(ObsSize*(1+index),0,left_down_S);
	m_S.extractMatrix(ObsSize*(1+index),ObsSize*(1+index),right_down_S);
	m_S.joinMatrix(left_up_S,right_up_S,left_down_S,right_down_S);

 }



