/* +---------------------------------------------------------------------------+
   |          The Mobile Robot Programming Toolkit (MRPT) C++ library          |
   |                                                                           |
   |                   http://mrpt.sourceforge.net/                            |
   |                                                                           |
   |   Copyright (C) 2005-2009  University of Malaga                           |
   |                                                                           |
   |    This software was written by the Perception and Robotics               |
   |      research group, University of Malaga (Spain).                        |
   |    Contact: Jose-Luis Blanco  <jlblanco@ctima.uma.es>                     |
   |                                                                           |
   |  This file is part of the MRPT project.                                   |
   |                                                                           |
   |     MRPT is free software: you can redistribute it and/or modify          |
   |     it under the terms of the GNU General Public License as published by  |
   |     the Free Software Foundation, either version 3 of the License, or     |
   |     (at your option) any later version.                                   |
   |                                                                           |
   |   MRPT is distributed in the hope that it will be useful,                 |
   |     but WITHOUT ANY WARRANTY; without even the implied warranty of        |
   |     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         |
   |     GNU General Public License for more details.                          |
   |                                                                           |
   |     You should have received a copy of the GNU General Public License     |
   |     along with MRPT.  If not, see <http://www.gnu.org/licenses/>.         |
   |                                                                           |
   +---------------------------------------------------------------------------+ */

#include <mrpt/core.h>
#include <mrpt/monoslam/CMonoSlam.h>
#include <mrpt/monoslam/CMonoSlamMotionModel.h>

using namespace std;
using namespace mrpt;
using namespace mrpt::utils;
using namespace mrpt::poses;
using namespace mrpt::slam;
using namespace mrpt::monoslam;
using namespace mrpt::math;
using namespace mrpt::gui;
using namespace mrpt::vision;
using namespace mrpt::hwdrivers;


/******************************************************************************************************************************/
/**
*   Entradas
* h_predicted = posicion esperada en coordenadas (u,v) de la caracteristicas que se ven (h_all)
* S_predicted = innovacion
* H_predicted = matriz de ecuaciones asociadas a las caracteristica
* std_pxl = desviacion ruido proceso (sigma_imageNoise de mmm)
* features (CFeaturePatch)
* cam = parametros intrinsecos de la camara (CCamModel)
* im = imagen actual
* chi2inv_table de mmm
*
*   Salidas
* h = coord (u,v) de las caracteristicas predichas que ademas se han emparejado correctamente (para el EKF)
* z       = coord (u,v) de las caracteristicas encontradas (MEDIDA) (para el EKF)
* matche_all = coord (u,v) de las caracteristicas encontradas (MEDIDA) (DE MOMENTO NO STA IMPLEMENTADO, SERVIRA POSTERIORMENTE PARA DIBUJAR)
* H_matched = matriz de ecuaciones de las caract emparejadas (para el EKF)
* R_matched = matriz de varianza del ruido del proceso		 (para el EKF)
*/

//									FOR EKF!!!
void  CMonoSlam::MSMatching(CMatrixDouble &h_predicted,
									  const CMatrixDouble &S_predicted,
									  CMatrixDouble &H_predicted,
									  mrpt::utils::CImage &im,
									  CMatrixDouble &hh,
									  CMatrixDouble &zz,
									  CMatrixDouble &H_matched,
									  CMatrixDouble &R_matched,
									  CFeaturePatch &features)
{
	mrpt::utils::CImageFloat ic(im);
//	ic.saveToTextFile("c:\\imgc.txt");

	float No_Sigma = 3;
	int halfPatchSize = 10;
	unsigned int predicted_size = (unsigned int )h_predicted.getRowCount();	//total number of features

    //predicted_image son las caracteristicas que se ven redondeadas al valor superior mas cercano
	CMatrixDouble predicted_image(h_predicted.getRowCount(),h_predicted.getColCount()),aux;
	predicted_image.ones();
	aux = h_predicted;
	aux.matrix_floor();
	predicted_image += aux;
	//std::cout<<"predicted_image:\n"<<predicted_image<<"\n";

	//mached_all es una matriz de num_carac x 2 inicializada a -1 para luego eliminar las caract no matches
	CMatrixDouble matched_all(predicted_size,2);
	m_MatchCoord.setSize(predicted_size,2);
	matched_all.ones();
	matched_all *= -1;
	m_MatchCoord = matched_all;

	CMatrixDouble S_i(2,2);
	CMatrixDouble eigvect(2,2);
	CMatrixDouble eigval(2,2);
	int halfW_x,halfW_y;
	size_t umax=0;
	size_t vmax=0;
	int index = 0;
	//double max_val=0;
	float max_val=0;
	mrpt::utils::CImage featurePatch;
	CMatrixDouble inv_S_i;
	CMatrixFloat  correlationScore;
	//CMatrix correlationScore;
	int j_k, i_k;

    
	// Para todas las caracteristicas que se ven se hace: (u,v)(Si-1)(u,v)'<=X^2(distancia mahalanobis)
	for(unsigned int i=0;i<predicted_size;i++)
	{
		if (h_predicted(i,0)>0)  //if pixel coord u  >0							
		{
			max_val=0;
			S_predicted.extractMatrix(2*i,2*i,S_i); //submatriz diagonal 2x2 asociadas a la caracteristicas (define la region eliptica de aceptacion)
			//S_predicted.extractMatrix(2*index,2*index,S_i); //submatriz diagonal 2x2 asociadas a la caracteristicas (define la region eliptica de aceptacion)
			index++;
		 
			S_i.force_symmetry();
			S_i.eigenVectors(eigvect,eigval);

			if(eigval(0,0)==0 || eigval(1,1)==0)
			{			
			    //std::cout<<"[Mono-Slam] S_predicted: \n"<<S_predicted<<"\n";			
				features.set_hits(i,features.get_hits(i)-1);		
				std::cout<<"[Mono-Slam] feature eigval(0,0)==0 || eigval(1,1)==0!  press any key ...\n";
  				std::cin.get();
 			        continue;

			}

			//ELLIPTICAL SEARCH
			inv_S_i.setSize(2,2);

			if((eigval(0,0)<0) || (eigval(1,1)<0))
			{
				halfW_x = (int)ceil(2*sqrt(S_i(0,0)));		//2veces la desviacion en x
				halfW_y = (int)ceil(2*sqrt(S_i(1,1)));		//2veces la desviacion en y
 			} 
			else
			{
				//mrpt::mathchol(S_i,inv_S_i);
				inv_S_i = S_i.inv();
				halfW_x = (int)ceil((No_Sigma / sqrt( inv_S_i(0,0) - ((inv_S_i(0,1) * inv_S_i(0,1)) / inv_S_i(1,1)) )));
				halfW_y = (int)ceil((No_Sigma / sqrt( inv_S_i(1,1) - ((inv_S_i(0,1) * inv_S_i(0,1)) / inv_S_i(0,0)) )));		
			}

			if ( m_iterations_count<3)
			{
				if ((halfW_x > (int)im.getWidth()/3)||(halfW_y > (int)im.getHeight()/3))
				{
						halfW_x = 20;		//2veces la desviacion en x
						halfW_y = 20;		//2veces la desviacion en y
				}

			}

			std::cout<<"\n[Mono-Slam] feature # "<<i<<" matching result: \neigenValues: \n"<<eigval<<"\n";
			//std::cout<<" S_i: \n"<<S_i<<"\n";			
			//std::cout<<"[Mono-Slam] halfW_x : "<<halfW_x<<"\n";
			//std::cout<<"[Mono-Slam] halfW_y : "<<halfW_y<<"\n";
			//std::cout<<"[Mono-Slam] S_predicted: \n"<<S_predicted<<"\n";


			j_k = round( predicted_image(i,0) );
			i_k = round( predicted_image(i,1) );

			//std::cout<<"j_k-halfW_x: "<<j_k-halfW_x<<"\n";
			//std::cout<<"i_k-halfW_y: "<<i_k-halfW_y<<"\n";
			//std::cout<<"j_k+halfW_x: "<<j_k+halfW_x<<"\n";
			//std::cout<<"i_k+halfW_y: "<<i_k+halfW_y<<"\n";
			//std::cout<<"imagen_row: "<<im.getHeight()<<"\n";
			//std::cout<<"imagen_col: "<<im.getWidth()<<"\n";
			//std::cout<<"patch_row: "<<featurePatch.getHeight()<<"\n";
			//std::cout<<"patch_col: "<<featurePatch.getWidth()<<"\n";

			// Para la region de aceptacion
			correlationScore.zeros();
			int left=j_k-halfW_x;
			int up=i_k-halfW_y;
			int long_x=2*halfW_x;
			int long_y=2*halfW_y;
			int temp=0;

			if ((halfW_x > (int)im.getWidth()/3)||(halfW_y > (int)im.getHeight()/3))	//Buscamos en toda la imagen
			{
				std::cout<<"feature search area is large!  press any key ...\n";

				std::cout<<"[Mono-Slam] halfW_x : "<<halfW_x<<"\n";
				std::cout<<"[Mono-Slam] halfW_y : "<<halfW_y<<"\n";
				std::cout<<"[Mono-Slam] S_i: \n"<<S_i<<"\n";			
				
				//std::cin.get();

    			//features.set_hits(i,-1);		
				features.set_hits(i,features.get_hits(i)-1);		

			}
			else
			{
				if	  (((j_k-halfW_x)>=halfPatchSize) && ((j_k+halfW_x)<=((int)cam.cam_ncols()-halfPatchSize))
					&& ((i_k-halfW_y)>=halfPatchSize) && ((i_k+halfW_y)<=((int)cam.cam_nrows()-halfPatchSize)))
				//nos aseguramos que no estemos en el borde de la imagen
				{
					//std::cout<<"Entra en la region de aceptacion para la caracteristica: "<<i<<"\n";
					features.get_patch(i,featurePatch);

					//im.cross_correlation_FFT(featurePatch,correlationScore,left,up,long_x,long_y);
					im.cross_correlation(featurePatch,correlationScore,left,up,long_x,long_y);
					correlationScore.find_index_max_value(umax,vmax, max_val);
					//im.openCV_cross_correlation(featurePatch,umax,vmax,max_val,left-10,up-10,long_x+20,long_y+20);
					if (0)
					{
						CMatrix A(correlationScore); A.adjustRange();
						CImageFloat imgf(A);
						CDisplayWindow	win("SCORE");
						win.showImage(imgf);
						win.waitForKey();
					}

				}
				else
				{//casos limite
					//std::cout<<"entra en casos especiales para la caracteristica: "<<i<<"\n";
					if ((j_k-halfW_x-halfPatchSize<0)&&(j_k+halfW_x-halfPatchSize>0))
					{
						left=halfPatchSize;
						long_x=j_k+halfW_x-halfPatchSize;
						temp++;
					}
					if ((j_k+halfW_x>(int)cam.cam_ncols()-halfPatchSize)&&(j_k-halfW_x < (int)cam.cam_ncols()-halfPatchSize))
					{
						long_x=cam.cam_ncols()-halfPatchSize-(j_k-halfW_x);
						temp++;
					}
					if ((i_k-halfW_y-halfPatchSize<0)&&(i_k+halfW_y-halfPatchSize>0))
					{
						up=halfPatchSize;
						long_y=i_k+halfW_y-halfPatchSize;
						temp++;
					}
					if ((i_k+halfW_y>(int)cam.cam_nrows()-halfPatchSize)&&(i_k-halfW_y < (int)cam.cam_nrows()-halfPatchSize))
					{
						long_y=cam.cam_nrows()-halfPatchSize-(i_k-halfW_y);
						temp++;
					}
					if (temp>0)
					{
						//std::cout<<"left: "<<left<<"\n";
						//std::cout<<"up: "<<up<<"\n";
						//std::cout<<"long_x: "<<long_x<<"\n";
						//std::cout<<"long_y: "<<long_y<<"\n";

						features.get_patch(i,featurePatch);
						im.cross_correlation(featurePatch, correlationScore,left , up , long_x , long_y);
						correlationScore.find_index_max_value(umax,vmax, max_val);
						//im.openCV_cross_correlation(featurePatch,umax,vmax,max_val,left-10,up-10,long_x+20,long_y+20);
					}

				}

//mrpt::utils::CImageFloat p(featurePatch);
//p.saveToTextFile("c:\\patchc.txt");
//std::cout<<"pausado en el patch\n";
//std::cin.get();

//std::cout<<"left: "<<left<<"	up: "<<up<<"	longx: "<<long_x<<"	longy: "<<long_y<<"\n";
//std::cout<<"col: "<<j_k<<"	row: "<<i_k<<"\n\n";

			//Select maximum correlation value
			//	umax=0;		vmax=0;		max_val=0;

//MRPT_OS::sprintf(nombre,500,"c:\\correlationScore_%02d.txt",i);
//correlationScore.saveToTextFile(nombre);

				//mahalanob_dist(j_k,i_k,halfW_x,halfW_y,inv_S_i,correlationScore);

				//correlationScore.find_index_max_value(umax,vmax,max_val);
				/*******************************************************************/
				//int um,vm;
				//float mv;
				//correlationScore.find_index_max_value(um,vm,mv);
				/*******************************************************************/
//std::cout<<"caracteristica: "<<i<<" um:   "<<um+left<<"    :      vm:   "<<vm+up<<"        : mv:      "<<mv<<"\n";
//std::cout<<"Matching results: "<<i<<" umax: "<<umax<<"    :      vmax: "<<vmax<<"        : max_val: "<<max_val<<"\n";
std::cout<<"Matching results: "<<i<<"        : max_val: "<<max_val<<"\n";


				if( max_val > options.match_threshold_up)	//si supera el umbral se actualiza el valor, sino se queda a -1
				{
					//matched_all(i,0) = (float)(um+left);
					//matched_all(i,1) = (float)(vm+up);
					matched_all(i,0) = (float)(umax+left);
					matched_all(i,1) = (float)(vmax+up);
				}
				else if ( max_val < options.match_threshold_down)				//La caracteristica esta oculta pero no es por mal tracking
				{
					features.set_hits(i,-1);	
					matched_all(i,0) = -2.0;
					matched_all(i,1) = -2.0;
			    	//std::cout<<"match val < match_threshold_down!  press any key ...\n";
					//std::cin.get();
				}
			}
		}
	}

	// Update hits of features (CFeaturePatch)
    for (unsigned int s=0 ; s < predicted_image.getRowCount() ; s++)
	{
		if (predicted_image(s,0) > 0)
		{
			if (matched_all(s,0) > -1)
			{
//				std::cout<<"caracteristica "<<s<<" se le SUMA un acierto\n";
                features.set_hits(s,1+features.get_hits(s));
			}
			else if (matched_all(s,0) == -1)
			{
//				std::cout<<"caracteristica "<<s<<" se le RESTA un acierto\n";
				features.set_hits(s,-1+features.get_hits(s));
			}
		}
	}

	//Construct results

	//z es vector columna para luego operar mas facilmente en EKF update
	zz.setSize(0,0);	//al final se redimensionan 'z' y 'h' para eliminar las caracteristicas no encontradas
	hh.setSize(0,0);
	
	m_z = m_h;

	unsigned int pos=0;
	for (unsigned int k=0 ; k < predicted_size ; k++)
	{
		if (matched_all(k,0) > -1)
		{
			pos=(unsigned int)zz.getRowCount();
			zz.setSize(pos+2,1);
			zz(pos,0)   = matched_all(k,0) + 0.5f;
			zz(pos+1,0) = matched_all(k,1) + 0.5f;

			m_z(k,0) = zz(pos,0);
			m_z(k,1) = zz(pos+1,0);

			//std::cout<<"xz : "<< z(pos,0) << "   yz : "<< z(pos+1,0)<<"  \n";

			hh.setSize(pos+2,1);
            hh(pos,0)   = h_predicted(k,0);
			hh(pos+1,0) = h_predicted(k,1);

 		}
	}

	H_matched.setSize(0,0);
	CMatrixDouble Hline(2,H_predicted.getColCount());
	index=0;
	for (unsigned int nr=0 ; nr < h_predicted.getRowCount() ; nr++)
	{
		if(h_predicted(nr,0) > -1)//if is visible
		{
			if (matched_all(nr,0) > -1)//if is matched
			{
				H_matched.setSize(2+H_matched.getRowCount(),H_predicted.getColCount());
				H_predicted.extractMatrix(2*index,0,Hline);
				H_matched.insertMatrix(H_matched.getRowCount()-2,0,Hline);
			}
			index++;
		}
	}

	R_matched.unit(zz.getRowCount());
	R_matched *= square( options.sigma_image_noise );


	m_MatchCoord = matched_all;
}

/**************************************************MAHALANOBIS DISTANCE************************************************/
void  CMonoSlam::mahalanob_dist(int j_k, int i_k, int halfW_x, int halfW_y, CMatrixDouble &inv_S, CMatrixDouble &correlationScore)
{
	if ((correlationScore.getRowCount()==(unsigned int)(2*halfW_x))&&(correlationScore.getColCount()==(unsigned int)(2*halfW_y)))
	{
		int j,i;
		CMatrixDouble	innov(1,2),meas;
		for  ( j = j_k - halfW_x ; j < j_k+halfW_x ; j++)
			for ( i = i_k - halfW_y ; i < i_k+halfW_y ; i++)
			{
				innov(0,0) = (float)(j - j_k);
				innov(0,1) = (float)(i - i_k);
				meas = innov * inv_S *~innov;
				if (meas(0,0) > 5.9915)
				{
					correlationScore(j + halfW_x - j_k, i + halfW_y - i_k) = 0.0;
				}
			}
	}
}

/********************************************************AMBIGUITY***********************************************************/
void  CMonoSlam::Ambiguity(CMatrixDouble	&HH,
									 CMatrixDouble	&RR,
									 CMatrixDouble	&hh,
									 CMatrixDouble	&zz)
{
	CVectorDouble	innov, innov_ordenada;
	size_t i;
	innov.resize(hh.getRowCount()/2);
	innov_ordenada.resize(hh.getRowCount()/2);

	if (innov_ordenada.size()>0)	// Added JLBC 07 , MAAA translated to MRPT 09
	{
		for ( i = 0; i < innov.size() ; i++)
			innov[i] = sqrt((zz(2*i,0)-hh(2*i,0))*(zz(2*i,0)-hh(2*i,0))+(zz(2*i+1,0)-hh(2*i+1,0))*(zz(2*i+1,0)-hh(2*i+1,0)));
		innov_ordenada = innov;

		/*Realizar la mediana de la innovacion*/
		std::sort(innov_ordenada.begin(),innov_ordenada.end());

		float mediana = innov_ordenada [round(innov_ordenada.size()/2)];

		if (mediana + 7 <innov_ordenada[innov_ordenada.size()-1])	//si hay al menos un valor con ambiguedad se hace el bucle
		{
			//Eliminamos todas las caractersticas cuya distancia sea mayor a la mediana + 5pixeles (este valor es heurstico)
			CMatrixDouble H(0,0),h(0,0),z(0,0), aux;
			for (i = 0 ; i < innov.size() ; i++)
			{
				if (innov[i] < mediana+7)
				{
					//Ajustamos los tamaos
					H.setSize(H.getRowCount()+2,HH.getColCount());
					h.setSize(h.getRowCount()+2,hh.getColCount());
					z.setSize(z.getRowCount()+2,zz.getColCount());
					//Cargamos los valores correctos
					HH.extractMatrix(2*i,0,2,(unsigned int)HH.getColCount(),aux);
					H.insertMatrix(H.getRowCount()-2,0,aux);

					hh.extractMatrix(2*i,0,2,(unsigned int)hh.getColCount(),aux);
					h.insertMatrix(h.getRowCount()-2,0,aux);

					zz.extractMatrix(2*i,0,2,(unsigned int)zz.getColCount(),aux);
					z.insertMatrix(z.getRowCount()-2,0,aux);
				}
			}

	/*		if (H.getRowCount() != HH.getRowCount())
			{
				std::cout<<"ENCONTRADA AMBIGUEDAD\n\n";
				std::cout<<"hh: "<<~hh<<"\n";
				std::cout<<"zz: "<<~zz<<"\n";
				std::cout<<"h: "<<~h<<"\n";
				std::cout<<"z: "<<~z<<"\n";
	//			std::cin.get();
			}
	*/		HH = H;
			hh = h;
			zz = z;

			RR.unit(h.getRowCount());
			RR *= square( options.sigma_image_noise );
		}
	}
}


void  CMonoSlam::Ambiguity(CMatrixDouble	&RR,
									CMatrixDouble	&hh,
									CMatrixDouble	&zz,
									CVectorDouble	&match)
{
	CVectorDouble	innov, innov_ordenada;
	size_t i;
	innov.resize(hh.getRowCount()/2);
	innov_ordenada.resize(hh.getRowCount()/2);

	if (innov_ordenada.size()>0)	 
	{

		for ( i = 0; i < innov.size() ; i++)
			innov[i] = sqrt((zz(2*i,0)-hh(2*i,0))*(zz(2*i,0)-hh(2*i,0))+(zz(2*i+1,0)-hh(2*i+1,0))*(zz(2*i+1,0)-hh(2*i+1,0)));
		innov_ordenada = innov;

		/*Realizar la mediana de la innovacion*/
		std::sort(innov_ordenada.begin(),innov_ordenada.end());

		float mediana = innov_ordenada [round(innov_ordenada.size()/2)];

		//std::cout<<"innov: "<<innov<<"\n";
		//std::cout<<"hh: "<<~hh<<"\n";
		//std::cout<<"zz: "<<~zz<<"\n";

		if (mediana + 7 <innov_ordenada[innov_ordenada.size()-1])	//si hay al menos un valor con ambiguedad se hace el bucle
		{
			CMatrixDouble h(0,0),z(0,0),aux;
			for (i = 0 ; i < innov.size() ; i++)
			{
				if (innov[i] < mediana+7)
				{
					h.setSize(h.getRowCount()+2,hh.getColCount());
					z.setSize(z.getRowCount()+2,zz.getColCount());

					hh.extractMatrix(2*i,0,2,(unsigned int)hh.getColCount(),aux);
					h.insertMatrix(h.getRowCount()-2,0,aux);

					zz.extractMatrix(2*i,0,2,(unsigned int)zz.getColCount(),aux);
					z.insertMatrix(z.getRowCount()-2,0,aux);
				}
				else
					match[i] = -1.0;
			}

	/*		if (h.getRowCount() != hh.getRowCount())
			{
				std::cout<<"ENCONTRADA AMBIGUEDAD\n\n";
				std::cout<<"hh: "<<~hh<<"\n";
				std::cout<<"zz: "<<~zz<<"\n";
				std::cout<<"h: "<<~h<<"\n";
				std::cout<<"z: "<<~z<<"\n";
	//			std::cin.get();
			}
	*/		hh = h;
			zz = z;

			RR.unit(h.getRowCount());
			RR *= square( options.sigma_image_noise );
		}
	}
}

/*****************************************************AJOGD********************************************************************/
//									FOR UKF and IEKFS!!!
void  CMonoSlam::MSMatching(CMatrixDouble &h_predicted,
									  CMatrixDouble &S_predicted,
									  mrpt::utils::CImage &im,
									  CMatrixDouble &hh,
									  CMatrixDouble &zz,
									  CMatrixDouble &R_matched,
									  mrpt::math::CVectorDouble &match,
									  CFeaturePatch &features)
{

	mrpt::utils::CImageFloat ic(im);
//	ic.saveToTextFile("c:\\imgc.txt");

	//if (features.size() != h_predicted.getRowCount())
	//		THROW_EXCEPTION("CMonoSlam MSMatching error. Features Wrong Dimension");

	if (features.size()==0 || h_predicted.getRowCount()==0)
		return;

//	float No_Sigma = 3;
	int halfPatchSize = 10;
	unsigned int predicted_size = (unsigned int )h_predicted.getRowCount();	//total number of features

    //predicted_image son las caracteristicas que se ven redondeadas al valor superior mas cercano
	CMatrixDouble predicted_image(h_predicted.getRowCount(),h_predicted.getColCount()),aux;
	predicted_image.ones();
	aux = h_predicted;
	aux.matrix_floor();
	predicted_image += aux;

	//mached_all es una matriz de num_carac x 2 inicializada a -1 para luego eliminar las caract no matches
	CMatrixDouble matched_all(predicted_size,2);
	matched_all.ones();
	matched_all *= -1;

	m_MatchCoord.setSize(predicted_size,2);
	matched_all.ones();
	matched_all *= -1;
	m_MatchCoord = matched_all;

	CMatrixDouble S_i(2,2);

	int halfW_x,halfW_y;
	size_t umax = 0;
	size_t vmax = 0;
	int index = 0;
	float max_val=0.0;

	mrpt::utils::CImage featurePatch;
	CMatrixDouble inv_S_i;
	CMatrixFloat correlationScore;
	int j_k, i_k;

	CMatrixDouble eigvect(2,2);
	CMatrixDouble eigval(2,2);
	float No_Sigma = 3;


	match.resize(0);

	// Para todas las caracteristicas que se ven se hace: (u,v)(Si-1)(u,v)'<=X^2(distancia mahalanobis)
	for(unsigned int i=0;i<predicted_size;i++)
	{
		if (h_predicted(i,0)>0)	//si coordenada 'u' >0
		{
			max_val=0;
			S_predicted.extractMatrix(2*i,2*i,S_i); //submatriz diagonal 2x2 asociadas a la caracteristicas (define la region eliptica de aceptacion)
			//S_predicted.extractMatrix(2*index,2*index,S_i); //submatriz diagonal 2x2 asociadas a la caracteristicas (define la region eliptica de aceptacion)
			index++;
		 
			S_i.force_symmetry();
			S_i.eigenVectors(eigvect,eigval);

			if(eigval(0,0)==0 || eigval(1,1)==0)
			{
			
			    //std::cout<<"[Mono-Slam] S_predicted: \n"<<S_predicted<<"\n";			
				continue;

			}

			//ELLIPTICAL SEARCH
			inv_S_i.setSize(2,2);

			if((eigval(0,0)<0) || (eigval(1,1)<0))
			{
				halfW_x = (int)ceil(2*sqrt(S_i(0,0)));		//2veces la desviacion en x
				halfW_y = (int)ceil(2*sqrt(S_i(1,1)));		//2veces la desviacion en y
 			} 
			else
			{
				//mrpt::mathchol(S_i,inv_S_i);
				inv_S_i = S_i.inv();
				halfW_x = (int)ceil((No_Sigma / sqrt( inv_S_i(0,0) - ((inv_S_i(0,1) * inv_S_i(0,1)) / inv_S_i(1,1)) )));
				halfW_y = (int)ceil((No_Sigma / sqrt( inv_S_i(1,1) - ((inv_S_i(0,1) * inv_S_i(0,1)) / inv_S_i(0,0)) )));		
			}

			if ( m_iterations_count<3)
			{
				if ((halfW_x > (int)im.getWidth()/3)||(halfW_y > (int)im.getHeight()/3))
				{
						halfW_x = 20;		//2veces la desviacion en x
						halfW_y = 20;		//2veces la desviacion en y
				}

			}

//std::cout<<"caract: "<<i<<" autovalores:\n"<<eigval<<"\n";
//std::cout<<"halfW_x: "<<halfW_x<<"\n";
//std::cout<<"halfW_y: "<<halfW_y<<"\n";

			j_k = round( predicted_image(i,0) );
			i_k = round( predicted_image(i,1) );

//			std::cout<<"j_k-halfW_x: "<<j_k-halfW_x<<"\n";
//			std::cout<<"i_k-halfW_y: "<<i_k-halfW_y<<"\n";
//			std::cout<<"j_k+halfW_x: "<<j_k+halfW_x<<"\n";
//			std::cout<<"i_k+halfW_y: "<<i_k+halfW_y<<"\n";
//			std::cout<<"imagen_row: "<<im.getHeight()<<"\n";
//			std::cout<<"imagen_col: "<<im.getWidth()<<"\n";
//			std::cout<<"patch_row: "<<featurePatch.getHeight()<<"\n";
//			std::cout<<"patch_col: "<<featurePatch.getWidth()<<"\n";

			// Para la region de aceptacion
			correlationScore.zeros();
			int left=j_k-halfW_x;
			int up=i_k-halfW_y;
			int long_x=2*halfW_x;
			int long_y=2*halfW_y;
			int temp=0;

			if ((halfW_x > (int)im.getWidth()/3)||(halfW_y > (int)im.getHeight()/3))	//Si la region de busqueda es muy grande eliminamos la caract
			{
				std::cout<<"feature search area is large!  press any key ...\n";
				//std::cin.get();
    			features.set_hits(i,-1);		
			}
			else
			{
				if	  (((j_k-halfW_x)>=halfPatchSize) && ((j_k+halfW_x)<=((int)cam.cam_ncols()-halfPatchSize))
					&& ((i_k-halfW_y)>=halfPatchSize) && ((i_k+halfW_y)<=((int)cam.cam_nrows()-halfPatchSize)))
				//nos aseguramos que no estemos en el borde de la imagen
				{
	//				std::cout<<"Entra en la region de aceptacion para la caracteristica: "<<i<<"\n";
					features.get_patch(i,featurePatch);
					im.cross_correlation(featurePatch,correlationScore,left,up,long_x,long_y);
					correlationScore.find_index_max_value(umax,vmax, max_val);
 					//im.openCV_cross_correlation(featurePatch,umax,vmax,max_val,left-10,up-10,long_x+20,long_y+20);
				}
				else
				{//casos limite
	//				std::cout<<"entra en casos especiales para la caracteristica: "<<i<<"\n";
					if ((j_k-halfW_x-halfPatchSize<0)&&(j_k+halfW_x-halfPatchSize>0))
					{
						left=halfPatchSize;
						long_x=j_k+halfW_x-halfPatchSize;
						temp++;
					}
					if ((j_k+halfW_x>(int)cam.cam_ncols()-halfPatchSize)&&(j_k-halfW_x < (int)cam.cam_ncols()-halfPatchSize))
					{
						long_x=cam.cam_ncols()-halfPatchSize-(j_k-halfW_x);
						temp++;
					}
					if ((i_k-halfW_y-halfPatchSize<0)&&(i_k+halfW_y-halfPatchSize>0))
					{
						up=halfPatchSize;
						long_y=i_k+halfW_y-halfPatchSize;
						temp++;
					}
					if ((i_k+halfW_y>(int)cam.cam_nrows()-halfPatchSize)&&(i_k-halfW_y < (int)cam.cam_nrows()-halfPatchSize))
					{
						long_y=cam.cam_nrows()-halfPatchSize-(i_k-halfW_y);
						temp++;
					}
					if (temp>0)
					{
	//					std::cout<<"left: "<<left<<"\n";
	//					std::cout<<"up: "<<up<<"\n";
	//					std::cout<<"long_x: "<<long_x<<"\n";
	//					std::cout<<"long_y: "<<long_y<<"\n";

						features.get_patch(i,featurePatch);
						im.cross_correlation(featurePatch,correlationScore,left,up,long_x,long_y);
						correlationScore.find_index_max_value(umax,vmax, max_val);
						//im.openCV_cross_correlation(featurePatch,umax,vmax,max_val,left-10,up-10,long_x+20,long_y+20);
					}
				}

//mrpt::utils::CImageFloat p(featurePatch);
//p.saveToTextFile("c:\\patchc.txt");
//std::cout<<"pausado en el patch\n";
//std::cin.get();

//std::cout<<"left: "<<left<<"	up: "<<up<<"	longx: "<<long_x<<"	longy: "<<long_y<<"\n";
//std::cout<<"col: "<<j_k<<"	row: "<<i_k<<"\n\n";

			//Select maximum correlation value
				//umax=0;		vmax=0;		max_val=0;
//MRPT_OS::sprintf(nombre,500,"c:\\correlationScore_%02d.txt",i);
//correlationScore.saveToTextFile(nombre);
			//correlationScore.find_index_max_value(umax,vmax,max_val);
//std::cout<<"caracteristica: "<<i<<"umax: "<<umax+left<<"    :      vmax: "<<vmax+up<<"        : max_val: "<<max_val<<"\n\n\n";
				if( max_val > options.match_threshold_up)		//si supera el umbral se actualiza el valor, sino se queda a -1
				{
					match.push_back(1.0);
 					//matched_all(i,0) = (float)(umax-1);
					//matched_all(i,1) = (float)(vmax-1);
					matched_all(i,0) = (float)(umax+left);
					matched_all(i,1) = (float)(vmax+up);
 				}
				else if ( max_val < options.match_threshold_down)				//La caracteristica esta oculta pero no es por mal tracking
				{
					matched_all(i,0) = -2.0;
					matched_all(i,1) = -2.0;
					match.push_back(-1.0);
				}
				else										//Caracteristica con mal tracking
					match.push_back(-1.0);
			}
		}
	}

//std::cout<<"matched_all: \n"<<matched_all<<"\n";


	// Update hits of features (CFeaturePatch)

    for (unsigned int s=0 ; s < predicted_image.getRowCount() ; s++)
	{
		if (predicted_image(s,0) > 0)
		{
			if (matched_all(s,0) > -1)
			{
//				std::cout<<"caracteristica "<<s<<" se le SUMA un acierto\n";
                features.set_hits(s,1+features.get_hits(s));
			}
			else if (matched_all(s,0) == -1)
			{
//				std::cout<<"caracteristica "<<s<<" se le RESTA un acierto\n";
				features.set_hits(s,-1+features.get_hits(s));
			}
		}
	}

	//Construct results
	zz.setSize(0,0);	//al final se redimensionan 'z' y 'h' para eliminar las caracteristicas no encontradas
	hh.setSize(0,0);
	m_z = m_h;

	unsigned int pos=0;
	for (unsigned int k=0 ; k < predicted_size ; k++)
	{
		if (matched_all(k,0) > -1)
		{
			pos=(unsigned int)zz.getRowCount();
			zz.setSize(pos+2,1);
			m_z(k,0) = zz(pos,0)   = matched_all(k,0) + 0.5f;
			m_z(k,1) = zz(pos+1,0) = matched_all(k,1) + 0.5f;

			hh.setSize(pos+2,1);
            hh(pos,0)   = h_predicted(k,0);
			hh(pos+1,0) = h_predicted(k,1);
		}
	}

	R_matched.unit(zz.getRowCount());
	R_matched *= square(options.sigma_image_noise);

	m_MatchCoord = matched_all;

}

