/* +---------------------------------------------------------------------------+
   |          The Mobile Robot Programming Toolkit (MRPT) C++ library          |
   |                                                                           |
   |                   http://mrpt.sourceforge.net/                            |
   |                                                                           |
   |   Copyright (C) 2005-2009  University of Malaga                           |
   |                                                                           |
   |    This software was written by the Perception and Robotics               |
   |      research group, University of Malaga (Spain).                        |
   |    Contact: Jose-Luis Blanco  <jlblanco@ctima.uma.es>                     |
   |                                                                           |
   |  This file is part of the MRPT project.                                   |
   |                                                                           |
   |     MRPT is free software: you can redistribute it and/or modify          |
   |     it under the terms of the GNU General Public License as published by  |
   |     the Free Software Foundation, either version 3 of the License, or     |
   |     (at your option) any later version.                                   |
   |                                                                           |
   |   MRPT is distributed in the hope that it will be useful,                 |
   |     but WITHOUT ANY WARRANTY; without even the implied warranty of        |
   |     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         |
   |     GNU General Public License for more details.                          |
   |                                                                           |
   |     You should have received a copy of the GNU General Public License     |
   |     along with MRPT.  If not, see <http://www.gnu.org/licenses/>.         |
   |                                                                           |
   +---------------------------------------------------------------------------+ */

#include <mrpt/core.h>
#include <mrpt/monoslam/CMonoSlam.h>
#include <mrpt/monoslam/CMonoSlamMotionModel.h>

using namespace std;
using namespace mrpt;
using namespace mrpt::utils;
using namespace mrpt::poses;
using namespace mrpt::slam;
using namespace mrpt::monoslam;
using namespace mrpt::math;
using namespace mrpt::gui;
using namespace mrpt::vision;
using namespace mrpt::hwdrivers;




//----------------------------------------------------------------
/* plot_results_2D */
//----------------------------------------------------------------

void  CMonoSlam::plot_results_2D(CMatrixDouble					&zz,
								CMatrixDouble		&hh,
								CMatrixDouble		&h,
								CMatrixDouble		&S,
								CMatrixDouble		 &XYZ,
								mrpt::utils::CImage			 &framecolor,
								unsigned int					type,
								CVectorDouble    	&match,
								bool							verbose,
								bool							savedatas)

{
	MRPT_TRY_START

 	if (options.verbose)
		std::cout<<"Coordinate of the features:\n";

	CMatrix S_i(2,2);
	CMatrixDouble S_ii(2,2);
	size_t i,j;

	int		temp=0;
	int		no_match=0;
	bool	out = false;

	//if (/*zz.getRowCount()==0 || hh.getRowCount()==0 || */ h.getRowCount()==0 )
	//	return;

	// Plot predicted and matched features:
	
	// Observed (hh)
 	for (j = 0 ; j < hh.getRowCount()/2 ; j++)
		framecolor.cross(round(hh(2*j,0)),round(hh(2*j+1,0)),0x00FF00,'+');

	// Matched features (zz)
	for (j = 0 ; j <  zz.getRowCount()/2 ; j++)
		framecolor.cross(round(zz(2*j,0)),round(zz(2*j+1,0)),0xFF0000,'x');

	////Total (h)
	//for (j = 0 ; j < h.getRowCount() ; j++)
	//	framecolor.cross(round(h(j,0)),round(h(j,1)),0x00FF00,'+');


	// Write the frame number:
	framecolor.selectTextFont("9x15B");
	framecolor.textOut(5,5, format("Frame #%04i",m_iterations_count), 0x808080 );
	

	if (type != 0) //If it is not Naive EKF
	{
		//std::cout<<"h: "<<h.getRowCount()<<","<<h.getColCount()<<"\n";
		//std::cout<<"XYZ: "<<XYZ.getRowCount()<<","<<XYZ.getColCount()<<"\n";
		//std::cout<<"zz: "<<zz.getRowCount()<<","<<zz.getColCount()<<"\n";
		//std::cout<<"S: "<<S.getRowCount()<<","<<S.getColCount()<<"\n";
		//std::cout<<"match: "<<match.size()<<"\n";

		for (i=0;i<match.size();i++)	//Total predictions
		{
			out = false;
			while (((unsigned int)temp<h.getRowCount()) && (out==false))	//No visible
			{
				if ((h(temp,0)<0)||(h(temp,1)<0))
				{
					//std::cout<<"temp: "<<temp<<"\n";
					if (options.verbose)
					{
						std::cout<<"x: "<<XYZ(temp,0)<<"	y: "<<XYZ(temp,1)<<"	z: "<<XYZ(temp,2)<<"	"<<m_FeatPatch.get_name(temp)<<"	NO VISIBLE\n";
					}
					temp++;
				}
				else
					out = true;
			}
			if (match[i]<0)		//NO matched
			{
				//std::cout<<"temp: "<<temp<<"\n";
				if (options.verbose)
				{
					std::cout<<"x: "<<XYZ(temp,0)<<"	y: "<<XYZ(temp,1)<<"	z: "<<XYZ(temp,2)<<"	"<<m_FeatPatch.get_name(temp)<<"	VISIBLE PERO NO MATCHING\n";
				}
				XYZ(temp,3) = 2;
				//std::cout<<"nomatch: "<<no_match<<"\n";
				no_match++;
				framecolor.rectangle(round(h(temp,0)-10),round(h(temp,1)-10),round(h(temp,0)+10),round(h(temp,1)+10),0,8);
				framecolor.textOut(round(h(temp,0)),round(h(temp,1)), format("%02d",m_FeatPatch.get_name(temp)) ,0x0000FF);
			}
			else				//matched
			{
				//std::cout<<"temp: "<<temp<<"\n";
				if (options.verbose)
				{
					std::cout<<"x: "<<XYZ(temp,0)<<"	y: "<<XYZ(temp,1)<<"	z: "<<XYZ(temp,2)<<"	"<<m_FeatPatch.get_name(temp)<<"	VISIBLE Y MATCHING\n";
				}
				XYZ(temp,3) = 1;
				//std::cout<<"i: "<<i<<"\n";
				S.extractMatrix(2*i,2*i,S_ii);
				S_i = S_ii;							//optimize
				S_i.force_symmetry();
				//std::cout<<"i-nomatch: "<<i-no_match<<"\n";
				framecolor.ellipseGaussian(&S_i,(float)(round(zz(2*(i-no_match),0))),(float)(round(zz(2*(i-no_match)+1,0))), 3, 0xFFFF00, 4);
				framecolor.textOut((round(zz(2*(i-no_match),0))),(round(zz(2*(i-no_match)+1,0))), format("%02d",m_FeatPatch.get_name(temp)) ,0xFF0000);
			}
			temp++;
		}

		//std::cout<<"end plot 2D\n";
		while ((unsigned int)(temp) < h.getRowCount())	//No visible
		{
			if ((h(temp,0)<0)||(h(temp,1)<0))
			{
				//std::cout<<"temp: "<<temp<<"\n";
				if (options.verbose)
				{
					std::cout<<"x: "<<XYZ(temp,0)<<"	y: "<<XYZ(temp,1)<<"	z: "<<XYZ(temp,2)<<"	"<<m_FeatPatch.get_name(temp)<<"	NO VISIBLE\n";
				}
				temp++;
			}
		}
	}
	else //EKF type==0 (Naive)
	{
		//for (i=0;i<zz.getRowCount()/2;i++)	//para todas las matching
		for (i=0;i<m_MatchCoord.getRowCount();i++)	//para todas las matching			
		{
			temp = i;
			//std::cout<<"i: "<<i<<"\n";
			//while (((fabs(h(temp,0)-hh(2*i,0))>0.01)||(fabs(h(temp,1)-hh(2*i+1,0))>0.01))||((h(temp,0)<0)&&(h(temp,1)<0)))	//VISIBLE(predicha) Y NO MATCHING o NO VISIBLE
			if(m_MatchCoord(i,0)<0)//No Matched  
			{
				//if ((h(temp,0)<0)&&(h(temp,1)<0))
				if ((h(temp,0)<0)&&(h(temp,1)<0))//No visible
				{
					XYZ(i,3) = 0;
					//std::cout<<"The no visible entry\n";
					if (options.verbose)
					{
						std::cout<<"x: "<<XYZ(temp,0)<<"	y: "<<XYZ(temp,1)<<"	z: "<<XYZ(temp,2)<<"	"<<m_FeatPatch.get_name(temp)<<"	NO VISIBLE\n";
					}
				}
				else //Visible then draw a black rectangle
				{
					//std::cout<<"no matched entry\n";
					if (options.verbose)
					{
						std::cout<<"x: "<<XYZ(temp,0)<<"	y: "<<XYZ(temp,1)<<"	z: "<<XYZ(temp,2)<<"	"<<m_FeatPatch.get_name(temp)<<"	VISIBLE PERO NO MATCHING\n";
					}
					XYZ(temp,3) = 2;
					//std::cout<<"coord NO MATCH: "<<h(temp,0)<<", "<<h(temp,1)<<"\n";std::cin.get();
					no_match++;
					//std::cout<<"coord NO MATCH: "<<h(temp,0)<<", "<<h(temp,1)<<"\n";
					framecolor.rectangle(round(h(temp,0)-10),round(h(temp,1)-10),round(h(temp,0)+10),round(h(temp,1)+10),0,8);
					framecolor.textOut(round(h(temp,0)),round(h(temp,1)),format("%02d",m_FeatPatch.get_name(temp)),0x0000FF);
				}
				//temp++;
				//std::cout<<"temp no visble o match: "<<temp<<"\n";
			}
			else//Its visible and matched
			{

				//Si sale del bucle es VISIBLE Y MATCHING
				if (options.verbose)
				{
					std::cout<<"x: "<<XYZ(temp,0)<<"	y: "<<XYZ(temp,1)<<"	z: "<<XYZ(temp,2)<<"	"<<m_FeatPatch.get_name(temp)<<"	VISIBLE Y MATCHING\n";
				}
				XYZ(temp,3) = 1;
				//S.extractMatrix(2*(i+no_match),2*(i+no_match),S_ii);
				S.extractMatrix(2*i,2*i,S_ii);
				S_i = S_ii;							//optimize
				S_i.force_symmetry();
				//framecolor.ellipseGaussian(&S_i,(float)(round(zz(2*i,0))),(float)(round(zz(2*i+1,0))), 3, 0xFFFF00, 4);
				//framecolor.textOut((round(zz(2*i,0))),(round(zz(2*i+1,0))),format("%02d",m_FeatPatch.get_name(temp)),0xFF0000);
				framecolor.ellipseGaussian(&S_i,(float)(round(m_z(i,0))),(float)(round(m_z(i,1))), 3, 0xFFFF00, 4);
				framecolor.textOut((round(m_z(i,0))),(round(m_z(i,1))),format("%02d",m_FeatPatch.get_name(temp)),0xFF0000);
				//temp++;
				//std::cout<<"temp visible: "<<temp<<"\n";

				//std::cout<<"[Mono-Slam] Plot2d S_i: \n"<<S_i<<"\n";
			}
		}

		////El resto de caractersticas pueden ser no visibles o no matching
		//for (i=temp ; i<h.getRowCount() ; i++)
		//{
		//	if ((h(i,0)<0)&&(h(i,1)<0))	//NO VISIBLE
		//	{
		//		if (options.verbose)
		//		{
		//			std::cout<<"x: "<<XYZ(i,0)<<"	y: "<<XYZ(i,1)<<"	z: "<<XYZ(i,2)<<"	"<<m_FeatPatch.get_name(i)<<"	NO VISIBLE\n";
		//		}
		//	}
		//	else	//VISIBLE PERO NO MATCHING
		//	{
		//		if (options.verbose)
		//		{
		//			std::cout<<"x: "<<XYZ(i,0)<<"	y: "<<XYZ(i,1)<<"	z: "<<XYZ(i,2)<<"	"<<m_FeatPatch.get_name(i)<<"	VISIBLE PERO NO MATCHING\n";
		//		}
		//		XYZ(i,3) = 2;
		//		//std::cout<<"coord NO MATCH: "<<h(i,0)<<", "<<h(i,1)<<"\n";std::cin.get();
		//		framecolor.rectangle(round(h(i,0)-10),round(h(i,1)-10),round(h(i,0)+10),round(h(i,1)+10),0,8);
		//		framecolor.textOut(round(h(i,0)),round(h(i,1)),format("%02d",m_FeatPatch.get_name(temp)),0x0000FF);
		//	}
		//}
		//std::cout<<"XYZ: \n"<<XYZ<<"\n";
	}

 
//	if (options.save_data_files)
//		framecolor.saveToFile("./monoSlamOut/FrameColor.bmp");

	// Save in "m_progress2D"
	m_progress2D.copyFastFrom(framecolor);

	MRPT_TRY_END
}

