/* +---------------------------------------------------------------------------+
   |          The Mobile Robot Programming Toolkit (MRPT) C++ library          |
   |                                                                           |
   |                   http://mrpt.sourceforge.net/                            |
   |                                                                           |
   |   Copyright (C) 2005-2009  University of Malaga                           |
   |                                                                           |
   |    This software was written by the Perception and Robotics               |
   |      research group, University of Malaga (Spain).                        |
   |    Contact: Jose-Luis Blanco  <jlblanco@ctima.uma.es>                     |
   |                                                                           |
   |  This file is part of the MRPT project.                                   |
   |                                                                           |
   |     MRPT is free software: you can redistribute it and/or modify          |
   |     it under the terms of the GNU General Public License as published by  |
   |     the Free Software Foundation, either version 3 of the License, or     |
   |     (at your option) any later version.                                   |
   |                                                                           |
   |   MRPT is distributed in the hope that it will be useful,                 |
   |     but WITHOUT ANY WARRANTY; without even the implied warranty of        |
   |     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         |
   |     GNU General Public License for more details.                          |
   |                                                                           |
   |     You should have received a copy of the GNU General Public License     |
   |     along with MRPT.  If not, see <http://www.gnu.org/licenses/>.         |
   |                                                                           |
   +---------------------------------------------------------------------------+ */

#include <mrpt/core.h>
#include <mrpt/monoslam/CMonoSlam.h>
#include <mrpt/monoslam/CMonoSlamMotionModel.h>

using namespace std;
using namespace mrpt;
using namespace mrpt::utils;
using namespace mrpt::poses;
using namespace mrpt::slam;
using namespace mrpt::monoslam;
using namespace mrpt::math;
using namespace mrpt::gui;
using namespace mrpt::vision;
using namespace mrpt::hwdrivers;


//----------------------------------------------------------------
/* plot_results_3D */
//----------------------------------------------------------------

void  CMonoSlam::plot_results_3D(CMatrixDouble &XYZ)
{
	MRPT_TRY_START

	//if (m_zz.getRowCount()==0 || m_hh.getRowCount()==0 ||  m_h.getRowCount()==0 )
	//	return;

	//unsigned int num_obj_visible = ((unsigned int)(m_progress3D)->size()-3)/2;	//camara,ejes y suelo no cuentan
	//m_num_obj_visible = m_ObservedfeatList.size() ;// Add camera
	unsigned int num_actual_feat = (unsigned int)XYZ.getRowCount();

	CVectorDouble xv;
	CMatrixDouble	PXYZ,cov3D;
	CQuaternionDouble q;
	double r,p,y;
	char n[50];
	size_t i;


	PXYZ = CMonoSlam::pxyztpl2pxyz();
	if (options.verbose)
        std::cout<<"[Mono-Slam] PXYZ: \n"<< PXYZ << "\n";

	CMonoSlam::get_xkk(xv);
	xv.extract_vector(3,q);
	q.rpy(r,p,y);

	CMatrixDouble giro(3,3), aux(3,3);
	giro(0,0)=1;
	giro(1,2)=-1;
	giro(2,1)=1;

	if (!m_progress3D.present())
		m_progress3D = mrpt::opengl::COpenGLScene::Create();

 	//Adding new features to the scene using the following algorithm


	if ( m_num_obj_visible < num_actual_feat )	//new features added
	{
		for(i = 0; i < (num_actual_feat-m_num_obj_visible) ; i++)
		{
			{  
				//CEllipsoid
 			    opengl::CEllipsoidPtr obj1 = opengl::CEllipsoid::Create();
				PXYZ.extractMatrix(3*(m_num_obj_visible+i),3*(m_num_obj_visible+i),3,3,aux);
  				cov3D = (giro * aux * ~giro);
				cov3D.force_symmetry();

				obj1->setCovMatrix(cov3D);
				obj1->setColor(0.0f,1.0f,0.0f);
				obj1->setLocation(-(float)xv[0],(float)xv[2],(float)xv[1]);
				obj1->setName( format("E%02d",m_FeatPatch.get_name(m_num_obj_visible+i)) );
				(m_progress3D)->insert(obj1);

				//CSphere
				opengl::CSpherePtr obj2 = opengl::CSphere::Create();
				obj2->setColor(1.0f,0.0f,0.0f);
 				obj2->setRadius(0.05f);
				obj2->setName( format("S%02d",m_FeatPatch.get_name(m_num_obj_visible+i)) );
				(m_progress3D)->insert(obj2);

				++m_num_obj_visible;
			}
		}
	}

	//Camera 
	 CRenderizablePtr	obj = (m_progress3D)->getByName("cam");
	if (obj!=NULL)		// Added JLBC 2007; MAAA translated to MRPT 09
	{
		CEllipsoidPtr	obj_cam = ( CEllipsoidPtr)obj;

 		CMatrixDouble	pkkg,pxx(3,3);
		CMonoSlam::get_pkk(pkkg);
		pkkg.extractMatrix(0,0,pxx);
		aux = giro * pxx* ~giro;
		aux.force_symmetry();
		//aux.force_pd(); <-- TIO!!!!???!?!
		obj_cam->enableShowName(true);
		obj_cam->setCovMatrix(aux);
 		obj_cam->setLocation((float)-xv[0], (float)xv[2], (float)xv[1]);
		obj_cam->setColor(0.0f,1.0f,1.0f,0.75f);
	}

 
	//Features
	CMatrixDouble aux2;
//	std::cout<<"XYZ size: "<<XYZ.getRowCount()<<"\n";
//	std::cout<<"feat size: "<<feats.size()<<"\n";

	for (i = 0 ; i < XYZ.getRowCount() ; i++)
	{
		//std::cout<<"caract: "<<i<<"	nombre: "<<feats.get_name(i)<<"\n";

		 os::sprintf(n,50,"S%02d",m_FeatPatch.get_name(i));
		 CRenderizablePtr	obj1 = (m_progress3D)->getByName(n);
		 if(obj1)
		 {
			obj1->setLocation( -XYZ(i,0),XYZ(i,2),XYZ(i,1));
			obj1->enableShowName(true);
			if (XYZ(i,3) == 0)		//No visible
 				 obj1->setColor(0.0f,0.0f,1.0f,.6f);
 			else if (XYZ(i,3) ==1)	//Visible y Matching
 				obj1->setColor(1.0f,0.0f,0.0f,.6f);
 			else					//Visible pero NO Matching
 				 obj1->setColor(0.0f,0.0f,0.0f,.6f);
 		 }
 
		 os::sprintf(n,50,"E%02d",m_FeatPatch.get_name(i));
		 CRenderizablePtr	 obj3 = (m_progress3D)->getByName(n);
		 if(obj3)
		 {
 			 opengl::CEllipsoidPtr obj2 = (CEllipsoidPtr)obj3;
			 if(obj2)
			 {
				PXYZ.extractMatrix(3*i,3*i,3,3,aux2);
				aux2 = giro * aux2 * ~giro;
				aux2.force_symmetry();
				//aux.force_pd();
	
				//std::cout<<"[Mono-Slam]aux2: \n"<<aux2<<"\n";

				obj2->setScale(10);
				obj2->setLocation( -XYZ(i,0),XYZ(i,2),XYZ(i,1));
				obj2->enableShowName(true);
				obj2->setCovMatrix(aux2);

				if (XYZ(i,3) == 0)		//No visible
 					 obj2->setColor(0.0f,0.0f,1.0f,0.85f);
 				else if (XYZ(i,3) ==1)	//Visible y Matching
 					obj2->setColor(1.0f,0.0f,0.0f,0.85f);
 				else					//Visible pero NO Matching
 					 obj2->setColor(0.0f,0.0f,0.0f,0.85f);				
			 }
  		 }//if(obj3)
	}//for

 
	MRPT_TRY_END
}


